# Copyright 2020, Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

package Google::Ads::GoogleAds::V14::Enums::OfflineUserDataJobMatchRateRangeEnum;

use strict;
use warnings;

use Const::Exporter enums => [
  UNSPECIFIED              => "UNSPECIFIED",
  UNKNOWN                  => "UNKNOWN",
  MATCH_RANGE_LESS_THAN_20 => "MATCH_RANGE_LESS_THAN_20",
  MATCH_RANGE_20_TO_30     => "MATCH_RANGE_20_TO_30",
  MATCH_RANGE_31_TO_40     => "MATCH_RANGE_31_TO_40",
  MATCH_RANGE_41_TO_50     => "MATCH_RANGE_41_TO_50",
  MATCH_RANGE_51_TO_60     => "MATCH_RANGE_51_TO_60",
  MATCH_RANGE_61_TO_70     => "MATCH_RANGE_61_TO_70",
  MATCH_RANGE_71_TO_80     => "MATCH_RANGE_71_TO_80",
  MATCH_RANGE_81_TO_90     => "MATCH_RANGE_81_TO_90",
  MATCH_RANGE_91_TO_100    => "MATCH_RANGE_91_TO_100"
];

1;
