# Copyright 2020, Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

package Google::Ads::GoogleAds::V14::Enums::DisplayUploadProductTypeEnum;

use strict;
use warnings;

use Const::Exporter enums => [
  UNSPECIFIED                   => "UNSPECIFIED",
  UNKNOWN                       => "UNKNOWN",
  HTML5_UPLOAD_AD               => "HTML5_UPLOAD_AD",
  DYNAMIC_HTML5_EDUCATION_AD    => "DYNAMIC_HTML5_EDUCATION_AD",
  DYNAMIC_HTML5_FLIGHT_AD       => "DYNAMIC_HTML5_FLIGHT_AD",
  DYNAMIC_HTML5_HOTEL_RENTAL_AD => "DYNAMIC_HTML5_HOTEL_RENTAL_AD",
  DYNAMIC_HTML5_JOB_AD          => "DYNAMIC_HTML5_JOB_AD",
  DYNAMIC_HTML5_LOCAL_AD        => "DYNAMIC_HTML5_LOCAL_AD",
  DYNAMIC_HTML5_REAL_ESTATE_AD  => "DYNAMIC_HTML5_REAL_ESTATE_AD",
  DYNAMIC_HTML5_CUSTOM_AD       => "DYNAMIC_HTML5_CUSTOM_AD",
  DYNAMIC_HTML5_TRAVEL_AD       => "DYNAMIC_HTML5_TRAVEL_AD",
  DYNAMIC_HTML5_HOTEL_AD        => "DYNAMIC_HTML5_HOTEL_AD"
];

1;
