# Copyright 2020, Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

package Google::Ads::GoogleAds::V13::Enums::AffiliateLocationPlaceholderFieldEnum;

use strict;
use warnings;

use Const::Exporter enums => [
  UNSPECIFIED    => "UNSPECIFIED",
  UNKNOWN        => "UNKNOWN",
  BUSINESS_NAME  => "BUSINESS_NAME",
  ADDRESS_LINE_1 => "ADDRESS_LINE_1",
  ADDRESS_LINE_2 => "ADDRESS_LINE_2",
  CITY           => "CITY",
  PROVINCE       => "PROVINCE",
  POSTAL_CODE    => "POSTAL_CODE",
  COUNTRY_CODE   => "COUNTRY_CODE",
  PHONE_NUMBER   => "PHONE_NUMBER",
  LANGUAGE_CODE  => "LANGUAGE_CODE",
  CHAIN_ID       => "CHAIN_ID",
  CHAIN_NAME     => "CHAIN_NAME"
];

1;
