# Copyright 2020, Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

package Google::Ads::GoogleAds::V14::Enums::AdGroupTypeEnum;

use strict;
use warnings;

use Const::Exporter enums => [
  UNSPECIFIED                     => "UNSPECIFIED",
  UNKNOWN                         => "UNKNOWN",
  SEARCH_STANDARD                 => "SEARCH_STANDARD",
  DISPLAY_STANDARD                => "DISPLAY_STANDARD",
  SHOPPING_PRODUCT_ADS            => "SHOPPING_PRODUCT_ADS",
  HOTEL_ADS                       => "HOTEL_ADS",
  SHOPPING_SMART_ADS              => "SHOPPING_SMART_ADS",
  VIDEO_BUMPER                    => "VIDEO_BUMPER",
  VIDEO_TRUE_VIEW_IN_STREAM       => "VIDEO_TRUE_VIEW_IN_STREAM",
  VIDEO_TRUE_VIEW_IN_DISPLAY      => "VIDEO_TRUE_VIEW_IN_DISPLAY",
  VIDEO_NON_SKIPPABLE_IN_STREAM   => "VIDEO_NON_SKIPPABLE_IN_STREAM",
  VIDEO_OUTSTREAM                 => "VIDEO_OUTSTREAM",
  SEARCH_DYNAMIC_ADS              => "SEARCH_DYNAMIC_ADS",
  SHOPPING_COMPARISON_LISTING_ADS => "SHOPPING_COMPARISON_LISTING_ADS",
  PROMOTED_HOTEL_ADS              => "PROMOTED_HOTEL_ADS",
  VIDEO_RESPONSIVE                => "VIDEO_RESPONSIVE",
  VIDEO_EFFICIENT_REACH           => "VIDEO_EFFICIENT_REACH",
  SMART_CAMPAIGN_ADS              => "SMART_CAMPAIGN_ADS",
  TRAVEL_ADS                      => "TRAVEL_ADS"
];

1;
